@@ -2,66 +2,56 @@
     Maintenance manager module
 """
 from datetime import datetime
-import json
-from sdc_helpers.query_helper import QueryHelper
+from sdc_helpers.models.client import Client
 import boto3
-import sdc_engine_helpers.helpers as helpers
-from sdc_engine_helpers.personalize.maintenance.date_utils import DateUtils
+from sdc_engine_helpers.date_utils import DateUtils
 from sdc_engine_helpers.personalize.maintenance.campaign_state_manager import CampaignStateManager
 from sdc_engine_helpers.personalize.maintenance.solution_state_manager import SolutionStateManager
+from sdc_engine_helpers.personalize.manager import Manager
 
 
-class MaintenanceManager:
+class MaintenanceManager(Manager):
     """
         Manage AWS Personalize solutions and campaigns
     """
     date_utils = None
-    query_helper = None
-    service = None
     lambda_client = None
     personalize = None
     solution_state_manager = None
     campaign_state_manager = None
 
-    def __init__(self):
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
         self.date_utils = DateUtils()
-        self.query_helper = QueryHelper()
         self.solution_state_manager = SolutionStateManager()
         self.campaign_state_manager = CampaignStateManager()
-        self.service = self.query_helper.get_service(slug='recommend')
         self.personalize = boto3.client('personalize')
 
-    def __del__(self):
-        del self.query_helper
-
     def perform_maintenance(self):
         """
             Create new Personalize solution versions and update Personalize campaigns
             when required
         """
-        # Get all active client subscriptions for the 'recommend' service
-        subscriptions_properties = self.query_helper.get_subscriptions_properties(
-            service_id=self.service['id']
-        )
-
-        for client_id, subscription_properties in subscriptions_properties.items():
+        # Get all active client subscriptions for the supplied service
+        for client in self.query_helper.get_clients():
             # Set up a flag to decide whether its necessary to update this subscription
             # after maintenance of this subscription
-            update_engine = False
+            update = False
 
-            client = self.query_helper.get_client(client_id=client_id)
-
-            # If the subscription contains solutions, perform solution maintenance
-            engine, engine_index = helpers.get_engine(
-                properties=subscription_properties,
-                slug='personalize',
-                return_index=True
+            subscription = self.query_helper.get_subscription(
+                client_id=client.id,
+                service_id=self.service.id,
+                from_cache=False
             )
 
-            if engine is None:
-                continue
+            solutions, engine_index = self.query_helper.get_engine_entity(
+                client_id=client.id,
+                service_id=self.service.id,
+                engine_slug=self.engine_slug,
+                entity_key='solutions',
+                from_cache=False
+            )
 
-            solutions = engine.get('solutions', None)
             if solutions is not None:
                 response = self.perform_solution_maintenance(
                     client=client,
@@ -69,12 +59,21 @@ class MaintenanceManager:
                 )
 
                 if response.get('solutions_updated') is True:
-                    engine['solutions'] = response.get('solutions')
-                    update_engine = True
+                    self.query_helper.update_solutions(
+                        subscription=subscription,
+                        engine_index=engine_index,
+                        value=response.get('solutions')
+                    )
+                    update = True
 
+            campaigns, engine_index = self.query_helper.get_engine_entity(
+                client_id=client.id,
+                service_id=self.service.id,
+                engine_slug=self.engine_slug,
+                entity_key='campaigns',
+                from_cache=False
+            )
 
-            # If the subscription contains campaigns, perform campaign maintenance
-            campaigns = engine.get('campaigns', None)
             if campaigns is not None:
                 response = self.perform_campaign_maintenance(
                     client=client,
@@ -82,23 +81,23 @@ class MaintenanceManager:
                 )
 
                 if response.get('campaigns_updated') is True:
-                    engine['campaigns'] = response.get('campaigns')
-                    update_engine = True
+                    self.query_helper.update_campaigns(
+                        subscription=subscription,
+                        engine_index=engine_index,
+                        value=response.get('campaigns')
+                    )
+                    update = True
 
-            if update_engine:
-                subscription_properties['engines'][engine_index] = engine
-                self.query_helper.update_subscription_properties(
-                    client_id=client_id,
-                    service_id=self.service['id'],
-                    properties=json.dumps(subscription_properties)
-                )
+            if update:
+                self.query_helper.update_subscription(subscription=subscription)
+                self.query_helper.flush_subscription_properties_cache(subscription=subscription)
 
-    def perform_solution_maintenance(self, *, client: dict, solutions: list) -> dict:
+    def perform_solution_maintenance(self, *, client: Client, solutions: list) -> dict:
         """
             Perform the required maintenance on a list of solutions
 
             args:
-                client (dict): The client this maintenance is being performed for
+                client (Client): The client this maintenance is being performed for
                 solutions (list): A list of solutions from the database to perform maintenance on
 
             returns:
@@ -122,7 +121,7 @@ class MaintenanceManager:
             if state.get('should_create_version') is True:
                 print(
                     'Building solution version for client: {client_name}'
-                    .format(client_name=client.get('name'))
+                    .format(client_name=client.name)
                 )
 
                 response = self.personalize.create_solution_version(
@@ -153,7 +152,7 @@ class MaintenanceManager:
                     ' to {status}'
                     .format(
                         arn=arn,
-                        client_name=client.get('name'),
+                        client_name=client.name,
                         status=new_version_status
                     )
                 )
@@ -166,12 +165,12 @@ class MaintenanceManager:
             'solutions': solutions
         }
 
-    def perform_campaign_maintenance(self, *, client: dict, campaigns: list) -> dict:
+    def perform_campaign_maintenance(self, *, client: Client, campaigns: list) -> dict:
         """
             Perform the required maintenance on a list of campaigns
 
             args:
-                client (dict): The client this maintenance is being performed for
+                client (Client): The client this maintenance is being performed for
                 campaigns (list): A list of campaigns from the database to perform maintenance on
 
             returns:
@@ -205,7 +204,7 @@ class MaintenanceManager:
                 print(
                     'Updating campaign for client: {client_name} to solution version: {arn}'
                     .format(
-                        client_name=client.get('name'),
+                        client_name=client.name,
                         arn=latest_solution_version_arn
                     )
                 )
@@ -235,7 +234,7 @@ class MaintenanceManager:
                     'Refreshing status on campaign: {arn} for client: {client_name} to {status}'
                     .format(
                         arn=arn,
-                        client_name=client.get('name'),
+                        client_name=client.name,
                         status=new_status
                     )
                 )
@@ -250,7 +249,7 @@ class MaintenanceManager:
                             'for client: {client_name}'
                             .format(
                                 arn=arn,
-                                client_name=client.get('name')
+                                client_name=client.name
                             )
                         )
 
